module utility_mod





    use param
    use cal
    use dal
    use sla     


!type CatalogueData
!        real(kind=double) :: RA
!        real(kind=double) :: dec
!        real(kind=single) :: error
!        real(kind=single) :: magnitude
!end type CatalogueData
  



contains


!************************************************************
! Subroutine HeapSort
! Sorts  one-dimensional array(1.. size) int ascending order.
! An index of the sorted order relative to the original
! order is returned.
!******************** 

subroutine HeapSort(array, index, size)
    integer, intent(in) :: size
    real(kind=double), intent(in), dimension(1 : size) :: array
    integer, intent(out), dimension(1 : size) :: index

    integer :: i, ir, j, l, irra
    double precision :: rra

    do i = 1, size
        index(i) = i
    end do   
  
    if (size .lt. 2) return


    l = size / 2 + 1
    ir = size   

    do 
        if (l .gt. 1) then
            l = l - 1
            rra = array(index(l))   
            irra = index(l)    
        else
            rra = array(index(ir))        
            irra = index(ir)
          
            index(ir) = index(1)
            ir = ir - 1
            
            if (ir .le. 1) then
             
                index(1) = irra
                exit
            end if
        end if

        i = l
        j = l + l
        do while(j .le. ir)
       
            if (j .lt. ir) then
                if(array(index(j)) .lt. array(index(j + 1))) j = j + 1
            end if
            if (rra .lt. array(index(j))) then
               
                index(i) = index(j)
                i = j
                j = j + j
            else
                exit
            end if
            
        end do   
          
        index(i) = irra
    end do
  
end subroutine HeapSort

!**********************************************************
! ReadInt8Column
!
!*********************************************************


subroutine ReadInt8Column(tableSet, heading, size, start, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading
    integer, intent(in) :: size, start
    integer(kind=int8), intent(inout), dimension(:) :: array
    integer(kind=int8), pointer :: i8col(:)
    type(ColumnT) :: pColumn
    integer i, k
    
    k = start
    pColumn = column(tableSet, heading, read)
    i8col => int8Data(pcolumn)
    do i = 1 , size
        k = k  + 1
        array(k) = i8col(i - 1)
    enddo
    call release(pColumn)

end subroutine ReadInt8Column
!**********************************************************
! ReadInt32Column
!
!*********************************************************
subroutine ReadInt32Column(tableSet, heading, size, start, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading
    integer, intent(in) :: size, start
    integer(kind=int32), intent(inout), dimension(:) :: array
    integer(kind=int32), pointer :: i32col(:)
    type(ColumnT) :: pColumn
    integer i, k
    
    k = start
    pColumn = column(tableSet, heading, read)
    i32col => int32Data(pcolumn)
    do i = 1 , size
        k = k  + 1
        array(k) = i32col(i - 1)
    enddo
    call release(pColumn)

end subroutine ReadInt32Column
!**********************************************************
! ReadReal32Column
!
!*********************************************************
subroutine ReadReal32Column(tableSet, heading, size, start, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading
    integer, intent(in) :: size, start
    real(kind=single), intent(inout), dimension(:) :: array
    real(kind=single), pointer :: real32Col(:)
    type(ColumnT) :: pColumn
    integer i, k

    pColumn = column(tableSet, heading, read)
    real32Col => real32Data(pcolumn)
    k = start
    do i = 1 , size
        k = k + 1   
        array(k) = real32Col(i - 1)
    enddo
    call release(pColumn)

end subroutine ReadReal32Column
!**********************************************************
! ReadReal64Column
!
!*********************************************************



subroutine ReadReal64Column(tableSet, heading, size, start, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading
    integer, intent(in) :: size, start
    real(kind=double), intent(inout), dimension(:) :: array
    real(kind=double), pointer :: real64Col(:)
    type(ColumnT) :: pColumn
    integer i, k
    
   
    pColumn = column(tableSet, heading, read)
    real64Col => real64Data(pcolumn)
    k = start
    do i = 1 , size
        k = k + 1
        array(k) = real64Col(i - 1)
    end do
    call release(pColumn)

end subroutine ReadReal64Column
!***************************************************
! AddReal32Cololumn
!***************************************************
subroutine AddReal32Column(tableSet, array, size, heading, units, comment, &
        nullValue, position)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, units, comment
    integer, intent(in) :: size
    real(kind=single), intent(in), dimension(:) :: array
    real(kind=single), intent(in) :: nullValue
    integer(kind=int32), intent(in), optional :: position
    real(kind=single), pointer :: real32Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)

   
    if (.not. hasColumn(tableSet, trim(heading))) then
        if (present(position)) then
            pColumn = addColumn(tableSet, trim(heading), real32, trim(units), ndims, trim(comment), position)
        else 
            pColumn = addColumn(tableSet, trim(heading), real32, trim(units), ndims, trim(comment))
        end if
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   


    real32Col => real32Data(pcolumn)

    do i = 1 , size
        if (array(i) .eq. nullValue) then
            call setToNull(pcolumn, i - 1)
        else
            real32Col(i - 1) = array(i)
        end if
  
    enddo
    call release(pcolumn)

end subroutine AddReal32Column

!***************************************************
! Add2Real32Column
! For 2-dimensional arrays
!***************************************************
subroutine Add2Real32Column(tableSet, heading, comment, &
        size, array, index, nullValue)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size, index
    real(kind=single), intent(in), dimension(:, :) :: array
    real(kind=single), intent(in) :: nullValue
    real(kind=single), pointer :: real32Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)
 


    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), real32, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   

    real32Col => real32Data(pcolumn)
  
    do i = 1 , size
        if (array(i, index) .eq. nullValue) then
            call setToNull(pcolumn, i - 1)
        else
            real32Col(i - 1) = array(i, index)
        end if
  
    enddo
    call release(pcolumn)

end subroutine Add2Real32Column


!***************************************************
! Add2Real32Column
! For 2-dimensional arrays
!***************************************************
subroutine Add2Real32IndexColumn(tableSet, heading, comment, &
        size, array, index, indexOrder, nullValue)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size, index, indexOrder(1 : size)
    real(kind=single), intent(in), dimension(:, :) :: array
    real(kind=single), intent(in) :: nullValue
    real(kind=single), pointer :: real32Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)
 



    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), real32, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   


    real32Col => real32Data(pcolumn)
  
    do i = 1 , size
        if (array(indexOrder(i), index) .eq. nullValue) then
            call setToNull(pcolumn, i - 1)
        else
            real32Col(i - 1) = array(indexOrder(i), index)
        end if
  
    enddo
    call release(pcolumn)

end subroutine Add2Real32IndexColumn


!***************************************************
! AddReal64Column
!***************************************************
subroutine AddReal64Column(tableSet, array, size, heading, units, comment, &
        nullValue, position)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, units, comment
    integer, intent(in) :: size

    real(kind=double), intent(in), dimension(1 : size) :: array
    real(kind=single), intent(in) :: nullValue
    integer(kind=int32), intent(in), optional :: position
    real(kind=double), pointer :: real64Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)



   


    if (.not. hasColumn(tableSet, trim(heading))) then
        if (present(position)) then
            pColumn = addColumn(tableSet, trim(heading), real64, trim(units), ndims, trim(comment), position)
        else 
            pColumn = addColumn(tableSet, trim(heading), real64, trim(units), ndims, trim(comment))
        end if
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   
   

    real64Col => real64Data(pcolumn)

    do i = 1 , size
        if (array(i) .eq. nullValue) then
            call setToNull(pcolumn, i - 1)
        else
            real64Col(i - 1) = array(i)
        end if
  
    enddo
    call release(pcolumn)

end subroutine AddReal64Column


!***************************************************
! AddReal64Column
!***************************************************
subroutine AddReal32IndexColumn(tableSet, heading, comment, &
        size, array, indexOrder, nullValue)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size, indexOrder(1 : size)

    real(kind=single), intent(in), dimension(1 : size) :: array
    real(kind=single), intent(in) :: nullValue
    real(kind=single), pointer :: real32Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)

    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), real32, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   
    real32Col => real32Data(pcolumn)

    do i = 1 , size
        if (array(indexOrder(i)) .eq. nullValue) then
            call setToNull(pcolumn, i - 1)
        else
            real32Col(i - 1) = array(indexOrder(i))
        end if
  
    enddo
    call release(pcolumn)

end subroutine AddReal32IndexColumn




!***************************************************
! AddReal64Column
!***************************************************
subroutine AddReal64IndexColumn(tableSet, heading, comment, &
        size, array, indexOrder, nullValue)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size, indexOrder(1 : size)

    real(kind=double), intent(in), dimension(1 : size) :: array
    real(kind=single), intent(in) :: nullValue
    real(kind=double), pointer :: real64Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)


    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), real64, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   

    real64Col => real64Data(pcolumn)

    do i = 1 , size
        if (array(indexOrder(i)) .eq. nullValue) then
            call setToNull(pcolumn, i - 1)
        else
            real64Col(i - 1) = array(indexOrder(i))
        end if
  
    enddo
    call release(pcolumn)

end subroutine AddReal64IndexColumn

!***************************************************
! AddInt8Cololumn
!***************************************************
subroutine AddInt8Column(tableSet, heading, comment, &
        size, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size
    integer(kind=int8), intent(in), dimension(:) :: array
    integer(kind=int8), pointer :: int8Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)

    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), integer8, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   


    int8Col => int8Data(pcolumn)

    forall (i = 1 : size) int8Col(i - 1) = array(i)
  
    call release(pcolumn)

end subroutine AddInt8Column




!***************************************************
! AddInt8Cololumn
!***************************************************
subroutine AddInt8IndexColumn(tableSet, heading, comment, &
        size, indexOrder, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size, indexOrder(1 : size)
    integer(kind=int8), intent(in), dimension(:) :: array
    integer(kind=int8), pointer :: int8Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)

    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), integer8, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   

    int8Col => int8Data(pcolumn)

    forall (i = 1 : size) int8Col(i - 1) = array(indexOrder(i))
  
    call release(pcolumn)

end subroutine AddInt8IndexColumn


!***************************************************
! AddInt32Cololumn
!***************************************************
subroutine AddInt32Column(tableSet, heading, comment, &
        size, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size
    integer(kind=int32), intent(in), dimension(:) :: array
    integer(kind=int32), pointer :: int32Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)

    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), integer32, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   
    !!! FIX ME!!!
    pcolumn = addColumn(tableSet, heading, integer32, comment, ndims)

    int32Col => int32Data(pcolumn)
  

    forall(i = 1: size) int32Col(i - 1) = array(i)

    call release(pcolumn)

end subroutine AddInt32Column


!***************************************************
! AddInt32Cololumn
!***************************************************
subroutine AddInt32IndexColumn(tableSet, heading, comment, &
        size, indexOrder, array)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size, indexOrder(1 : size)
    integer(kind=int32), intent(in), dimension(:) :: array
    integer(kind=int32), pointer :: int32Col(:)
    type(ColumnT) :: pColumn
    integer i
    integer :: ndims(1) = (/1/)


    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), integer32, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   
  
    int32Col => int32Data(pcolumn)

    forall(i = 1: size) int32Col(i - 1) = array(indexOrder(i))

    call release(pcolumn)

end subroutine AddInt32IndexColumn

!*******************************************************
! ArcLength - returns the distance (arc secs) between 2
! sources with coords (ra1, dec1) & (ra2, dec2)
! Returns the distance in arc seconds.
!*******************************************************
real(kind=double) function ArcLength(ra1, dec1, ra2, dec2)
    real(kind=double)              ::  ra1, ra2
    real(kind=double)              ::  dec1, dec2
    real(kind=double)              :: ang
    real(kind=double), parameter   :: PI = 3.14159265358979323846_double
    real(kind=double), save :: constant =  3600.0_double * 180.0_double / PI
    real(kind=double) :: cosDist
    ang = dabs(ra2 - ra1)
    if (ang > pi) ang = ang - pi


   if( cosdist > 0.9 ) cosdist = min(1.0d0, cosdist) 
          if( cosdist < -0.9 ) cosdist = max(-1.0d0, cosdist) 
     
    cosDist = dsin(dec1) * dsin(dec2) + dcos(dec1) &
                * dcos(dec2) * dcos(ang) 

    cosDist = max(-1.0_double, cosDist)
    cosDist = min(1.0_double,  cosDist)
    
    ArcLength = dacos(cosDist) * constant 

end function ArcLength
!*******************************************
! Function ComputeEpoch
!******************************************
real(kind=double) function  ComputeEpoch(dateObs)
   
    character(len = 19), intent(in)  :: dateObs               
    real(kind = double)  :: JulianDate0, JulianDate
    character(len = 19)  :: missionReferenceTime = "1998-01-01T00:00:00"
    
write(*, *) "Date string=", dateObs
    ! Compute the Julian date of the mission reference time.
    JulianDate0 = ModifiedJulianDate(missionReferenceTime)

    ! Compute the Julian date of the observation
    JulianDate = ModifiedJulianDate(dateObs)

    
    ComputeEpoch = 1998.0_double + (JulianDate - JulianDate0) / 365.25_double

end function ComputeEpoch
!*******************************************************
! ModifiedJulianDate - Returns the modified Julian date
! from the string date representation
!*******************************************************
real(kind=double) function ModifiedJulianDate(dateString)
    character(len = *), intent(in) :: dateString

    integer :: year            ! Stores the year from the date string
    integer :: month           ! Stores the month from the date string
    integer :: day             ! Stores the day from the date string  
    integer :: hour            ! Stores the hour from the date string
    integer :: minute          ! Stores the minute from the date string  
    integer :: second          ! Stores the second from the date string
    character(len = 1) char    ! Used for date and time extraction 
    ! from string representation
    double precision JulianDay    
    integer :: flag

    read(dateString, '(i4, a1, i2, a1, i2, a1, i2, a1, i2, a1, i2)') &
         year, char, month, char, day, char, hour, &
         char, minute, char, second     

    ! Get the modified Julian date for this date
    call sla_CLDJ(year, month, day, JulianDay, flag)

    ! Need to add on the fraction of a day corresponding to the time
    ModifiedJulianDate = JulianDay + (dble(hour) + &
         (dble(minute) + dble(second) / 60.0d0) / 60.0d0) / 24.0d0
end function ModifiedJulianDate

!**********************************
! ComputeAverage
! Calculates the weighted average and
! standard deviation
!**********************************
subroutine ComputeAverage(data, error, size, mean, standardDeviation)
    integer, intent(in) :: size        
    real(kind=double), intent(in), dimension(1 : size) :: data, error    
    real(kind=double), intent(out) :: mean, standardDeviation
    real(kind=double) :: weight
    real(kind=double) :: sum1, sum2, sum3
    real(kind=double) :: d, difference
    logical :: useWeights
    integer i

    useWeights = .true.
    weight = 1.0_double

    if (size .eq. 1) then
        mean = data(1)
        standardDeviation = error(1)
        return
    end if

    sum1 = 0.0_double
    sum2 = 0.0_double
    sum3 = 0.0_double
 

    do i = 1, size
        if (error(i) .le. 0.0) then
            useWeights = .false.
            sum1 = dble(size)
            sum3 = dble(size)
            exit
        end if
    end do 

    
    do i = 1, size
        if (useWeights) then
            weight = 1.0_double / (error(i) * error(i))
            sum1 = sum1 + weight
            sum2 = sum2 + data(i) * weight
            sum3 = sum3 + weight * weight
        else
            sum2 = sum2 + data(i)
          
        end if

    end do

    mean = sum2 / sum1  ! The mean

    sum2 = 0.0d0 
    do i = 1, size
        difference = data(i) - mean       
        if (useWeights) then
            weight = 1.0_double / (error(i) * error(i))
            sum2 = sum2 + weight * difference * difference
        else
            sum2 = sum2 + difference * difference
        end if 
    end do
    if (useWeights) then
        d = sum1 - sum3 / sum1
    else
        d = dble(size - 1)
    end if
    standardDeviation = dsqrt(sum2 / d) ! The standard deviation

end subroutine ComputeAverage



!***************************************************************
! Function MedianValue
! Find the median of X(1), ... , X(N), using as much of the quicksort
! algorithm as is needed to isolate it.
! N.B. On exit, the array X is partially ordered.
!**********************************************************************
real(kind=single) Function MedianValue(data, n)

    IMPLICIT NONE

    INTEGER, INTENT(IN)                :: n
    REAL(kind=double), INTENT(IN), DIMENSION(1 : n) :: data
    
    !************************************
    ! Local variables
    !***********************************

    REAL(kind=double)    :: temp, xhi, xlo, xmax, xmin
    LOGICAL :: odd
    INTEGER :: hi, lo, nby2, nby2p1, mid, i, j, k
    real(kind=double), dimension(:), allocatable :: x

  
    nby2 = n / 2
    nby2p1 = nby2 + 1
    odd  = .true.

!     HI & LO are position limits encompassing the median.

    IF (n == 2 * nby2) odd = .false.
    lo = 1
    hi = n
    IF (n < 3) THEN
        IF (n < 1) THEN
            MedianValue = 0.0_double
            RETURN
        END IF
        MedianValue = data(1)
        IF (n == 1) RETURN
        MedianValue = 0.5 * (MedianValue + data(2))
        RETURN
    END IF


    allocate(x(1 : n))
    x = data




!     Find median of 1st, middle & last values.

10 mid = (lo + hi)/2
    MedianValue = x(mid)
    xlo = x(lo)
    xhi = x(hi)
    IF (xhi < xlo) THEN          ! Swap xhi & xlo
        temp = xhi
        xhi = xlo
        xlo = temp
    END IF
    IF (MedianValue > xhi) THEN
        MedianValue = xhi
    ELSE IF (MedianValue < xlo) THEN
        MedianValue = xlo
    END IF

! The basic quicksort algorithm to move all values <= the sort key (XMED)
! to the left-hand end, and all higher values to the other end.

    i = lo
    j = hi
50  DO
        IF (x(i) >= MedianValue) EXIT
        i = i + 1
    END DO
    DO
        IF (x(j) <= MedianValue) EXIT
        j = j - 1
    END DO
    IF (i < j) THEN
        temp = x(i)
        x(i) = x(j)
        x(j) = temp
        i = i + 1
        j = j - 1

!     Decide which half the median is in.

        IF (i <= j) GO TO 50
    END IF

    IF (.NOT. odd) THEN
        IF (j .eq. nby2 .AND. i .eq. nby2p1) GO TO 130
        IF (j < nby2) lo = i
        IF (i > nby2p1) hi = j
        IF (i /= j) GO TO 100
        IF (i == nby2) lo = nby2
        IF (j == nby2p1) hi = nby2p1
    ELSE
        IF (j < nby2p1) lo = i
        IF (i > nby2p1) hi = j
        IF (i /= j) GO TO 100

! Test whether median has been isolated.

        IF (i .eq. nby2p1) then
            deallocate(x)
            RETURN
        end if
    END IF
100 IF (lo .lt. hi - 1) GO TO 10

    IF (.NOT. odd) THEN
        MedianValue = 0.5 * (x(nby2) + x(nby2p1))
        deallocate(x)
        RETURN
    END IF
    temp = x(lo)
    IF (temp .gt. x(hi)) THEN
        x(lo) = x(hi)
        x(hi) = temp
    END IF
    MedianValue = x(nby2p1)
    deallocate(x)
    RETURN

    !***************************************************************
    ! Special case, N even, J = N/2 & I = J + 1, so the median is
    ! between the two halves of the series.   Find max. of the first
    ! half & min. of the second half, then average.
    !***************************************************************
    130 xmax = x(1)
    DO k = lo, j
        xmax = max(xmax, x(k))
    END DO
    xmin = x(n)

    DO k = i, hi
        xmin = min(xmin, x(k))
    END DO

    MedianValue = 0.5 * (xmin + xmax)
    deallocate(x)
END function MedianValue

!***************************************************
! AddReal642DColumn
!***************************************************
subroutine AddReal642DColumn(tableSet, array, size, heading, comment)
    type(TableT), intent(in) :: tableSet
    character(len=*), intent(in) :: heading, comment
    integer, intent(in) :: size

    real(kind=double), intent(in), dimension(1 : size, 1 : 2) :: array
    
    real(kind=double), pointer :: real64Col(:, :)
    type(ColumnT) :: pColumn
    integer i, j
    integer :: ndims(2)

    ndims(1) = 2
    ndims(2) = size

    if (.not. hasColumn(tableSet, trim(heading))) then
        pColumn = addColumn(tableSet, trim(heading), real64, trim(comment), ndims)
    else
        pColumn = column(tableSet, trim(heading), modify)
    end if
   

!    real64Col => real64VectorData(pcolumn)

    real64Col => real64Array2Data(pColumn, 0)
    forall (j = 1 : 2)
        forall (i = 1 : size) real64Col(i - 1, j - 1) = array(i, j)
    end forall

    do i = 1, size
        write(*, *) i, array(i, 1), array(i, 2)
    end do


  
    call release(pColumn)

end subroutine AddReal642DColumn
!***************************************************
! IncreaseTableSize
!***************************************************
subroutine IncreaseTableSize(tableSet, nRows)
    type(TableT), intent(in) :: tableSet
    integer, intent(in) :: nRows
    integer :: noOfRows
   
    noOfRows = numberOfRows(tableSet)
    call insertRows(tableSet, noOfRows, nRows)
    
end subroutine IncreaseTableSize

!***************************************************
! AddToReal64Column
!***************************************************
!subroutine AddToReal64Column(tableSet, array, jStart, size, heading)
!    type(TableT), intent(in) :: tableSet
!    character(len=*), intent(in) :: heading
!    integer, intent(in) :: size, jStart!!
!
!    real(kind=double), intent(in), dimension(1 : size) :: array
!    
!    real(kind=double), pointer :: real64Col(:)
!    type(ColumnT) :: pColumn
!    integer i
!!    integer :: ndims(1) = (/1/)!
!!
!
!    pColumn = column(tableSet, trim(heading), modify)
!    
!    real64Col => real64Data(pcolumn)!
!
!    forall (i = 1 : size) real64Col(jStart + i - 1) = array(i)
!
!  
!    call release(pcolumn)
!
!end subroutine AddToReal64Column


end module utility_mod

